==> fr.ac6.linux.utils_1.0.0.201706270959.jar: resources/scripts/packaging_utils.sh <==
#
# Library of shell functions to install packages
#
# The kind of packages to install is set by the PACKAGE_INSTALL_TYPE environment variable
#		values can be tarball, ipkg, deb or rpm
#		Note:
#			Only tarball is currently supported and PACKAGE_INSTALL_TYPE is ignored
#
# The available functions are:
#
#	load_packages		Load a set of packages, if they exist in the platform package feed
#	require_packages	Load a set of packages that must exist in the platform package feed 
#
# WARNING: PKG_INSTALL_LOC should be set to SYSROOT_LOC or ROOTFS_LOC before sourcing this file
#
: ${debug_pkg_install:=false}
$debug_pkg_install && print_traces pkg_install
pkg_dbg() {
	dbg pkg_install "$@"
}
pkg_dbg_cat() {
	dbg_cat pkg_install "$@"
}

if [ -z "$PKG_INSTALL_LOC" ]; then
	echo "Package installation directory is not specified..."
	exit 1
fi
mkdir -p "$PKG_INSTALL_LOC"

if [ -z "$tempDir" ]; then
	tempDir=/tmp/populate-$package_name.$$
fi
mkdir -p "$tempDir"

loaded_tarballs=$PKG_INSTALL_LOC/etc/loaded.tarballs
mkdir -p $loaded_tarballs

#
# Search for package tarballs
#
require_script "version_utils.sh"

search_tarballs() {
	local required=$1
	shift
	
	local pkg_spec pkg_id
	for pkg_spec; do
		for pkg_id in $(search_all_versions $pkg_spec); do
			local pkg_name=${pkg_id%%_*}
			local pkg_vers=${pkg_id#*_}
			local pkg_arch=${pkg_vers#*_}
			pkg_vers=${pkg_vers%%_*}
			local found=false
			local recipe_id recipe_id_file pkg_id
			recipe_id_file=$basePackagesDir/$pkg_arch/${pkg_id}
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for tarball in $baseTarballsDir/$pkg_arch/${recipe_id}/${pkg_id}.tar*; do
					 if [ -r "$tarball" ]; then
					 	found=true
					 	break
					 fi
					# Note that going here is more an INTERNAL ERROR than anything else...
				done
			fi
		done
		if $found; then
			echo $tarball
		elif $required; then
			echo "Can't find required package '$pkg'" >&2
			return 1
		else
			echo "Skipping unknown package '$pkg'" >&2
		fi
	done
}

search_all_versions() {
	local pkg="$1"
	local pkg_name=${pkg%%_*}
	local pkg_vers pkg_rev pkg_arch
	if [ "$pkg_name" = "$pkg" ]; then
		pkg_vers='*'
		pkg_arch=
	else
		pkg_vers=${pkg#*_}
		pkg_arch=${pkg_vers#*_}
		if [ "$pkg_arch" = "$pkg_vers" ]; then
			pkg_arch=''
		fi
		pkg_vers=${pkg_vers%_$pkg_arch}
		if [ -n "$pkg_vers" ]; then
			pkg_rev=${pkg_vers#*-}
			if [ "$pkg_rev" = "$pkg_vers" ]; then
				pkg_rev='*'
			else
				pkg_rev="-$pkg_rev"
				pkg_vers=${pkg_vers%%-*}
			fi
		else
			pkg_vers='*'
			pkg-rev=''
		fi
	fi
	local pkg_archs
	if [ -z "$pkg_arch" ]; then
		pkg_archs="$package_arch all any"
	else
		pkg_vers=${pkg_vers%%_*}
		pkg_archs="$pkg_arch"
	fi

	local recipe_id recipe_id_file pkg_id version
	for arch in $pkg_archs; do
		pkg_id="${pkg_name}_${pkg_vers}${pkg_rev}_${arch}"
		for recipe_id_file in $basePackagesDir/$arch/${pkg_id}; do
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for tarball in $baseTarballsDir/$arch/${recipe_id}/${pkg_id}.tar*; do
					 if [ -r "$tarball" ]; then
					 	version=${tarball##*/}
					 	echo ${version%.tar*}
					 fi
				done
			fi
		done
	done | sort -rV | uniq
}

do_load_packages() {
	local pkg pkgs
	local status=0 
	local required="$1"
	shift

	pkgs=$(search_tarballs $required "$@"; status=$?)
	if [ $status -ne 0 ]; then return $status; fi
	if [ -n "$pkgs" ]; then
		load_tarballs $required $pkgs
	fi
}

require_packages() {
	do_load_packages true "$@"
}

load_packages() {
	do_load_packages false "$@"
}

do_load_depends() {
	local required=$1
	local depfile deplist status=0
	for depfile; do
		if [ -r "$depfile" ]; then
			deplist=$(cat $depfile)
			pkg_dbg "Loading dependencies from '$depfile':"
			pkg_dbg_cat "Loading $(basename "$depfile")" "$depfile"
			if do_load_packages $required $(parse_depend $required $deplist); then
				continue
			fi
			status=1
		fi
	done
	return $status
}

require_depends() {
	do_load_depends true "$@"
}

load_depends() {
	do_load_depends false "$@"
}

parse_package() {
	# Note that only ${package_arch} may contain underscores and that package_version cannot contain dashes
	local pkgTarball="$(basename $1)" 			# ${package_name}_${package_version}-${package_revision}_${package_arch}.tar*
	local bin_package="${pkgTarball%.tar*}" 	# ${package_name}_${package_version}-${package_revision}_${package_arch}
	local package=${bin_package/_/:}			# ${package_name}:${package_version}-${package_revision}_${package_arch}
	local package_arch=${package#*_}			# ${package_arch}
	local package=${bin_package%%_*}			# ${package_name}:${package_version}-${package_revision}
	local pkgvers=${package#*:}					# ${package_version}-${package_revision}
	local package_version=${pkgvers%%-*}		# ${package_version}
	local package_revision=${pkgvers#*-}		# ${package_revision}
	local package_name=${package%:*}			# ${package_name}
	echo $package_name $package_version $package_revision $package_arch
}

load_tarballs() {
	local required="$1"
	shift
	local pkgTarball
	for pkgTarball; do
		if [ -r "$pkgTarball" ]; then
			local bin_package="${pkgTarball%.tar*}"
			bin_package="${bin_package##*/}"
			if [ ! -r $allPackagesDir/$bin_package ]; then
				if $required; then
					echo "Required package '$bin_package' not found.'"
					return 1
				fi
				continue
			fi
			
			# Record we are loading it
			if [ -r "$tempDir/$bin_package" ]; then
				continue # This package was already considered this run
			fi
			touch "$tempDir/$bin_package"
			
			local recipe_id=$(cat $allPackagesDir/$bin_package)
			if [ "$loaded_tarballs/$recipe_id/$bin_package" -nt "$pkgTarball" ]; then
				continue # This package was already loaded in the sysroot and is up-to-date
			fi
		
#			local package_name package_version package_revision package_arch
#			parse_package $bin_package | read package_name package_version package_revision package_arch
#			local pkgtype=${package_name##*-}
#			local pkgname=${package_name%-*}
#			local package_id="${package_name}_${package_version}-${package_revision}"
			
			# First load depending packages
			local suff depfile
			for suff in rdepends auto-depends; do
				depfile="${allMetadataDir}/$recipe_id/${bin_package}.$suff"
				if [ -s "$depfile" ]; then
					if require_depends "$depfile"; then 
						true # Dependents correctly loaded
					else 
						# Dependencies not satisfied, can't load
						rm -f "$tempDir/$bin_package"
						if $required; then
							echo "Required package '$bin_package' not loaded due to missing dependencies.'" >&2
							return 1
						else
							log "Optional package '$bin_package' not loaded due to missing dependencies.'"
							return 0
						fi
					fi
				fi
			done
			
			# Then load the required package
			mkdir -p "$loaded_tarballs/$recipe_id"
			log -n "tar xvf '$pkgTarball' -C '$PKG_INSTALL_LOC' > '$loaded_tarballs/$recipe_id/$bin_package'"
			if tar xvf "$pkgTarball" -C "$PKG_INSTALL_LOC" > "$loaded_tarballs/$recipe_id/$bin_package"; then 
				log " Done." 
			else
				log " ERROR!..."
				echo "Can't extract required package '$bin_package' from tarball" >&2
				rm -f "$tempDir/$bin_package"
				return 1
			fi
		
			# Patch libtool auxiliary files to reflect their real position when installed in a sysroot
			# so that they refer to files installed in the same sysroot, not on the final root filesystem
			if [ "$PKG_INSTALL_LOC" != "$ROOTFS_LOC" ]; then
				local ifile
				for ifile in $(grep '\.l[ao]$' "$loaded_tarballs/$recipe_id/$bin_package"); do
					sed -e 's|\([[:space:]'"'"'=]\)/usr/|\1'"$PKG_INSTALL_LOC"'/usr/|g' \
					    -e 's|\([[:space:]'"'"'=]\)/lib/|\1'"$PKG_INSTALL_LOC"'/lib/|g' \
					    "$PKG_INSTALL_LOC/$ifile" > "$PKG_INSTALL_LOC/$ifile.new"
					if cmp -s "$PKG_INSTALL_LOC/$ifile" "$PKG_INSTALL_LOC/$ifile.new"; then
						rm -f "$PKG_INSTALL_LOC/$ifile.new"
					else
						rm -f "$PKG_INSTALL_LOC/$ifile"
						mv "$PKG_INSTALL_LOC/$ifile.new" "$PKG_INSTALL_LOC/$ifile"
						log "Patched '$ifile'"
					fi
				done
			fi
		
			# Add recommended packages
			for suff in rrecommends; do
				# These may fail, this is only a recommendation
				pkg_dbg_cat "Load depends from ${bin_package}.$suff" "${metadataDir}/${bin_package}.$suff"
				load_depends "${metadataDir}/${bin_package}.$suff"
			done
		
			# And finally add associated sub-packages
			local pkgbase=${bin_package%%_*}
			local pkgvers=${bin_package#*_}
			local derived_pkgs=
			for subpkg in $rootfs_optional_subpackages; do
				derived_pkgs="$derived_pkgs ${pkgbase}-${subpkg}_${pkgvers}"
			done
			pkg_dbg "Load associated subpackages: '$derived_pkgs'"
			load_packages $derived_pkgs
		else
			echo "INTERNAL ERROR on '$bin_package': Can't find package tarball '$pkgTarball'" >&2
			return 1
		fi
	done
	return 0
}

suppress_loaded_packages() {
	local loadedPkgs="$@"
	local loadedPkg
	for loadedPkg in $loadedPkgs; do
		local bin_package=$(basename $loadedPkg)
		if [ -s "$loadedPkg" ]; then
			log "Suppressing '$bin_package' from '$PKG_INSTALL_LOC'"
			(cd $PKG_INSTALL_LOC; rm -f $(cat "$loadedPkg"))
		fi
		rm -f "$loadedPkg"
	done
}

suppress_packages() {
	local binpkgs="$@"
	local binpkg
	local loaded_pkgs=
	
	for binpkg in $binpkgs; do
		for loaded_pkg in "$loaded_tarballs/*/$binpkg"; do
			if [ -r "$loaded_pkg" ]; then
				loaded_pkgs="$loaded_pkgs $loaded_pkg"
			fi
		done
	done
	suppress_loaded_packages $loaded_pkgs
}

wipe_recipes_output() {
	local recipe_id
	for recipe_id; do
		if [ -d "$loaded_tarballs/$recipe_id" ]; then
			suppress_loaded_packages "$loaded_tarballs/$recipe_id/*"
		fi
	done
}

==> fr.ac6.linux.utils_1.0.0.202110251403.jar: resources/scripts/packaging_utils.sh <==
#
# Library of shell functions to install packages
#
# The kind of packages to install is set by the PACKAGE_INSTALL_TYPE environment variable
#		values can be tarball, ipkg, deb or rpm
#		Note:
#			Only tarball is currently supported and PACKAGE_INSTALL_TYPE is ignored
#
# The available functions are:
#
#	load_packages		Load a set of packages, if they exist in the platform package feed
#	require_packages	Load a set of packages that must exist in the platform package feed 
#
# WARNING: PKG_INSTALL_LOC should be set to SYSROOT_LOC or ROOTFS_LOC before sourcing this file
# It may be changed afterwards by calling set_package_install_directory
#
: ${debug_pkg_install:=false}
$debug_pkg_install && print_traces pkg_install
pkg_dbg() {
	dbg pkg_install "$@"
}
pkg_dbg_cat() {
	dbg_cat pkg_install "$@"
}

set_package_install_directory() {
	PKG_INSTALL_LOC="$1"
	if [ -z "$PKG_INSTALL_LOC" ]; then
		internal_error 1 "Package installation directory is not specified..."
	fi
	mkdir -p "$PKG_INSTALL_LOC"
	
	if [ -z "$tempDir" ]; then
		tempDir=/tmp/populate-$package_name.$$
	fi
	mkdir -p "$tempDir"
	
	loaded_tarballs=$PKG_INSTALL_LOC/etc/loaded.tarballs
	mkdir -p $loaded_tarballs
}
set_package_install_directory "$PKG_INSTALL_LOC"

#
# Search for package tarballs
#
require_script "version_utils.sh"

search_tarballs() {
	local required=$1
	shift
	
	local pkg_spec pkg_id
	for pkg_spec; do
		for pkg_id in $(search_all_versions $pkg_spec); do
			local pkg_name=${pkg_id%%_*}
			local pkg_vers=${pkg_id#*_}
			local pkg_arch=${pkg_vers#*_}
			pkg_vers=${pkg_vers%%_*}
			local found=false
			local recipe_id recipe_id_file pkg_id
			recipe_id_file=$basePackagesDir/$pkg_arch/${pkg_id}
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for tarball in $baseTarballsDir/$pkg_arch/${recipe_id}/${pkg_id}.tar*; do
					 if [ -r "$tarball" ]; then
					 	found=true
					 	break
					 fi
					# Note that going here is more an INTERNAL ERROR than anything else...
				done
			fi
		done
		if $found; then
			echo $tarball
		elif $required; then
			echo "Can't find required package '$pkg'" >&2
			return 1
		else
			echo "Skipping unknown package '$pkg'" >&2
		fi
	done
}

search_preinsts() {
	local required=$1
	shift
	
	local pkg_spec pkg_id
	for pkg_spec; do
		for pkg_id in $(search_all_versions $pkg_spec); do
			local pkg_name=${pkg_id%%_*}
			local pkg_vers=${pkg_id#*_}
			local pkg_arch=${pkg_vers#*_}
			pkg_vers=${pkg_vers%%_*}
			local found=false
			local recipe_id recipe_id_file pkg_id
			recipe_id_file=$basePackagesDir/$pkg_arch/${pkg_id}
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for preinst in $baseMetadataDir/$pkg_arch/${recipe_id}/${pkg_id}.preinst; do
					 if [ -r "$preinst" ]; then
					 	found=true
					 	break
					 fi
					# Note that going here is more an INTERNAL ERROR than anything else...
				done
			fi
		done
		if $found; then
			echo $preinst
		fi
	done
}

search_postinsts() {
	local required=$1
	shift
	
	local pkg_spec pkg_id
	for pkg_spec; do
		for pkg_id in $(search_all_versions $pkg_spec); do
			local pkg_name=${pkg_id%%_*}
			local pkg_vers=${pkg_id#*_}
			local pkg_arch=${pkg_vers#*_}
			pkg_vers=${pkg_vers%%_*}
			local found=false
			local recipe_id recipe_id_file pkg_id
			recipe_id_file=$basePackagesDir/$pkg_arch/${pkg_id}
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for postinst in $baseMetadataDir/$pkg_arch/${recipe_id}/${pkg_id}.postinst; do
					 if [ -r "$postinst" ]; then
					 	found=true
					 	break
					 fi
					# Note that going here is more an INTERNAL ERROR than anything else...
				done
			fi
		done
		if $found; then
			echo $postinst
		fi
	done
}

search_all_versions() {
	local pkg="$1"
	local pkg_name=${pkg%%_*}
	local pkg_vers pkg_rev pkg_arch
	if [ "$pkg_name" = "$pkg" ]; then
		pkg_vers='*'
		pkg_arch=
	else
		pkg_vers=${pkg#*_}
		pkg_arch=${pkg_vers#*_}
		if [ "$pkg_arch" = "$pkg_vers" ]; then
			pkg_arch=''
		fi
		pkg_vers=${pkg_vers%_$pkg_arch}
		if [ -n "$pkg_vers" ]; then
			pkg_rev=${pkg_vers#*-}
			if [ "$pkg_rev" = "$pkg_vers" ]; then
				pkg_rev='*'
			else
				pkg_rev="-$pkg_rev"
				pkg_vers=${pkg_vers%%-*}
			fi
		else
			pkg_vers='*'
			pkg-rev=''
		fi
	fi
	local pkg_archs
	if [ -z "$pkg_arch" ]; then
		pkg_archs="$package_arch all any"
	else
		pkg_vers=${pkg_vers%%_*}
		pkg_archs="$pkg_arch"
	fi

	local recipe_id recipe_id_file pkg_id version
	for arch in $pkg_archs; do
		pkg_id="${pkg_name}_${pkg_vers}${pkg_rev}_${arch}"
		for recipe_id_file in $basePackagesDir/$arch/${pkg_id}; do
			if [ -s "$recipe_id_file" ]; then
				recipe_id=$(cat $recipe_id_file)
				for tarball in $baseTarballsDir/$arch/${recipe_id}/${pkg_id}.tar*; do
					 if [ -r "$tarball" ]; then
					 	version=${tarball##*/}
					 	echo ${version%.tar*}
					 fi
				done
			fi
		done
	done | sort -rV | uniq
}

do_load_packages() {
	local pkg pkgs
	local status=0 
	local required="$1"
	shift
	
	preinsts=$(search_preinsts $required "$@"; status=$?)
	pkgs=$(search_tarballs $required "$@"; status=$?)
	postinsts=$(search_postinsts $required "$@"; status=$?)
	if [ $status -ne 0 ]; then return $status; fi
	
	if [ -n "$preinsts" ]; then
		exec_scripts_on_system $preinsts
	fi
	
	if [ -n "$pkgs" ]; then
		load_tarballs $required $pkgs
	fi
	
	if [ -n "$postinsts" ]; then
		exec_scripts_on_system $postinsts
	fi
}

exec_scripts_on_system() {
	# Tell the system to run into staging sysroot
	UA_SYSROOT=1
	FAKECHROOT="fakechroot"
	export UA_SYSROOT FAKECHROOT
	exec $preinsts
}

require_packages() {
	do_load_packages true "$@"
}

load_packages() {
	do_load_packages false "$@"
}

do_load_depends() {
	local required=$1
	local depfile deplist status=0
	for depfile; do
		if [ -r "$depfile" ]; then
			deplist=$(cat $depfile)
			pkg_dbg "Loading dependencies from '$depfile':"
			pkg_dbg_cat "Loading $(basename "$depfile")" "$depfile"
			local parsed_depends=$(parse_depend $required $deplist)
			if [ $? -eq 0 ]; then
				if do_load_packages $required $parsed_depends; then
					continue
				fi
			fi
			status=1
		fi
	done
	return $status
}

require_depends() {
	do_load_depends true "$@"
}

load_depends() {
	do_load_depends false "$@"
}

parse_package() {
	# Note that only ${package_arch} may contain underscores and that package_version cannot contain dashes
	local pkgTarball="$(basename $1)" 			# ${package_name}_${package_version}-${package_revision}_${package_arch}.tar*
	local bin_package="${pkgTarball%.tar*}" 	# ${package_name}_${package_version}-${package_revision}_${package_arch}
	local package=${bin_package/_/:}			# ${package_name}:${package_version}-${package_revision}_${package_arch}
	local package_arch=${package#*_}			# ${package_arch}
	local package=${bin_package%%_*}			# ${package_name}:${package_version}-${package_revision}
	local pkgvers=${package#*:}					# ${package_version}-${package_revision}
	local package_version=${pkgvers%%-*}		# ${package_version}
	local package_revision=${pkgvers#*-}		# ${package_revision}
	local package_name=${package%:*}			# ${package_name}
	echo $package_name $package_version $package_revision $package_arch
}

load_tarballs() {
	local required="$1"
	shift
	local pkgTarball
	for pkgTarball; do
		if [ -r "$pkgTarball" ]; then
			local bin_package="${pkgTarball%.tar*}"
			bin_package="${bin_package##*/}"
			if [ ! -r $allPackagesDir/$bin_package ]; then
				if $required; then
					echo "Required package '$bin_package' not found.'"
					return 1
				fi
				continue
			fi
			
			# Record we are loading it
			if [ -r "$tempDir/$bin_package" ]; then
				continue # This package was already considered this run
			fi
			touch "$tempDir/$bin_package"
			
			local recipe_id=$(cat $allPackagesDir/$bin_package)
			if [ "$loaded_tarballs/$recipe_id/$bin_package" -nt "$pkgTarball" ]; then
				continue # This package was already loaded in the sysroot and is up-to-date
			fi
		
#			local package_name package_version package_revision package_arch
#			parse_package $bin_package | read package_name package_version package_revision package_arch
#			local pkgtype=${package_name##*-}
#			local pkgname=${package_name%-*}
#			local package_id="${package_name}_${package_version}-${package_revision}"
			
			# First load depending packages
			local suff depfile
			for suff in rdepends auto-depends; do
				depfile="${allMetadataDir}/$recipe_id/${bin_package}.$suff"
				if [ -s "$depfile" ]; then
					if require_depends "$depfile"; then 
						true # Dependents correctly loaded
					else 
						# Dependencies not satisfied, can't load
						rm -f "$tempDir/$bin_package"
						if $required; then
							echo "Required package '$bin_package' not loaded due to missing dependencies.'" >&2
							return 1
						else
							log "Optional package '$bin_package' not loaded due to missing dependencies.'"
							return 0
						fi
					fi
				fi
			done
			
			# Then load the required package
			mkdir -p "$loaded_tarballs/$recipe_id"
			log -n "tar xvf '$pkgTarball' -C '$PKG_INSTALL_LOC' --delay-directory-restore > '$loaded_tarballs/$recipe_id/$bin_package'"
			if tar xvf "$pkgTarball" -C "$PKG_INSTALL_LOC" --delay-directory-restore > "$loaded_tarballs/$recipe_id/$bin_package"; then 
				# Ensure that, if we install new elements in the native sysroot, they are visible to the shell;
				# to be deterministic, we must rehash the PATH each time this occurs
				hash -r
				log " Done." 
			else
				log " ERROR!..."
				rm -f "$tempDir/$bin_package"
				fatal 1 "Can't extract required package '$bin_package' from tarball" >&2
			fi
		
			# Patch libtool auxiliary files to reflect their real position when installed in a sysroot
			# so that they refer to files installed in the same sysroot, not on the final root filesystem
			if [ "$PKG_INSTALL_LOC" != "$ROOTFS_LOC" ]; then
				local ifile
				for ifile in $(grep '\.l[ao]$' "$loaded_tarballs/$recipe_id/$bin_package"); do
					sed -e 's|\([[:space:]'"'"'=]\)/usr/|\1'"$PKG_INSTALL_LOC"'/usr/|g' \
					    -e 's|\([[:space:]'"'"'=]\)/lib/|\1'"$PKG_INSTALL_LOC"'/lib/|g' \
					    "$PKG_INSTALL_LOC/$ifile" > "$PKG_INSTALL_LOC/$ifile.new"
					if cmp -s "$PKG_INSTALL_LOC/$ifile" "$PKG_INSTALL_LOC/$ifile.new"; then
						rm -f "$PKG_INSTALL_LOC/$ifile.new"
					else
						rm -f "$PKG_INSTALL_LOC/$ifile"
						mv "$PKG_INSTALL_LOC/$ifile.new" "$PKG_INSTALL_LOC/$ifile"
						log "Patched '$ifile'"
					fi
				done
			fi
		
			# Add recommended packages
			for suff in rrecommends; do
				# These may fail, this is only a recommendation
				pkg_dbg_cat "Load depends from ${bin_package}.$suff" "${metadataDir}/${bin_package}.$suff"
				load_depends "${metadataDir}/${bin_package}.$suff"
			done
		
			# And finally add associated sub-packages
			local pkgbase=${bin_package%%_*}
			local pkgvers=${bin_package#*_}
			local derived_pkgs=
			for subpkg in $rootfs_optional_subpackages; do
				derived_pkgs="$derived_pkgs ${pkgbase}-${subpkg}_${pkgvers}"
			done
			pkg_dbg "Load associated subpackages: '$derived_pkgs'"
			load_packages $derived_pkgs
		else
			echo "INTERNAL ERROR on '$bin_package': Can't find package tarball '$pkgTarball'" >&2
			return 1
		fi
	done
	return 0
}

suppress_loaded_packages() {
	local lock
	if [ X"$1" = X"-l" ]; then
		lock=$2
		shift 2
	fi
	local loadedPkgs="$@"
	local loadedPkg
	for loadedPkg in $loadedPkgs; do
		local bin_package=$(basename $loadedPkg)
		if [ -s "$loadedPkg" ]; then
			log "Suppressing '$bin_package' from '$PKG_INSTALL_LOC'"
			if [ -n "$lock" ]; then
				silent_do_locked $lock "(cd $PKG_INSTALL_LOC; xargs -a $loadedPkg -n 64 rm -d -f 2>&1 | grep -v 'Directory not empty')"
			else
				silent_do "(cd $PKG_INSTALL_LOC; xargs -a $loadedPkg -n 64 rm -d -f 2>&1 | grep -v 'Directory not empty')"
			fi
			# Beware that here we may suppress content from the native sysroot that may have been hashed by bash
			# as the native sysroot is in the PATH; we must then reset the internal bash hash of the PATH content
			# or we may try to execute some removed command (this is the case for example when populating 
			# util-linux-native where you reinstall flock while needing it...)
			hash -r
		fi
		rm -f "$loadedPkg"
	done
}

suppress_packages() {
	local binpkgs="$@"
	local binpkg lock
	local loaded_pkgs=
	if [ X"$1" = X"-l" ]; then
		lock=$2
		shift 2
	fi
	
	for binpkg in $binpkgs; do
		for loaded_pkg in "$loaded_tarballs/*/$binpkg"; do
			if [ -r "$loaded_pkg" ]; then
				loaded_pkgs="$loaded_pkgs $loaded_pkg"
			fi
		done
	done
	if [ -n "$lock" ]; then
		suppress_loaded_packages -l $lock $loaded_pkgs
	else
		suppress_loaded_packages $loaded_pkgs
	fi
}

wipe_recipes_output() {
	local recipe_spec recipe_id lock
	if [ X"$1" = X"-l" ]; then
		lock=$2
		shift 2
	fi
	for recipe_id; do # Beware these IDs may be globbing patterns...
		for recipe_spec in $loaded_tarballs/$recipe_id; do
			if [ -d "$recipe_spec" ]; then
				if [ -n "$lock" ]; then
					suppress_loaded_packages -l "$lock" "$recipe_spec/*"
				else
					suppress_loaded_packages "$recipe_spec/*"
				fi
			fi
		done
	done
}
